package componentImage;

//THIS CLASS WAS MODIFIED FROM TREEFORM ORIGINAL (SEE LICENSE INFO BELOW)

//TreeForm Syntax Tree Drawing Software
//Copyright (C) 2006  Donald Derrick 
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either version 2
//of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
//package userInterface;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

public class SaveComponentImage {

	public String exportComponentImage(Component comp) {
		return exportComponentImage(
				comp, new Rectangle(0, 0, (int) comp.getBounds().getWidth(), (int) comp.getBounds().getHeight())
		);
	}

	public String exportComponentImage(Component comp, Rectangle rect) {
		return exportComponentImage(comp, rect, null, null);
	}

	public String exportComponentImage(Component comp, Rectangle rect, String dialogTitle, String fileName) {		
		String currentPath = null;
		try {
			currentPath = new File(".").getCanonicalPath();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		return exportComponentImage(comp, rect, dialogTitle, fileName, currentPath, comp);
	}

	/**
	 * @param comp - the java.awt.Component whose image we are capturing
	 * @param rect - the Rectangle whose dimensions we are using to crop the image
	 * @param dialogTitle - the title of the JFileChooser dialog that will appear
	 * @param defaultFileName - the default filename that will be provided to the JFileChooser dialog that will appear
	 * @param defaultOpenLocation - the directory the JFileChooser will first offer the user
	 * @param compToOverlap - the Component over which the JFileChooser will open 
	 * 
	 * This method opens the file chooser, and asks the user to select the export
	 * type and type in a file name.  If the file name is valid, the project is
	 * exported.
	 **/
	public String exportComponentImage(Component comp, Rectangle rect, String dialogTitle, String defaultFileName, String defaultOpenLocation, Component compToOverlap) {
		JFileChooser lChooser = new JFileChooser(defaultOpenLocation);
		String dt = "Save Image";
		if(dialogTitle != null) {
			dt = dialogTitle;
		}
		lChooser.setDialogTitle(dt);
		lChooser.setAcceptAllFileFilterUsed(false);
		String fn = "image-1";
		if(defaultFileName != null) {
			fn = defaultFileName;
		}
		lChooser.setSelectedFile(new File(fn));
		lChooser.setFileFilter(new FileFilterJPG300());

		lChooser.addChoosableFileFilter(new FileFilterJPG600());
		lChooser.addChoosableFileFilter(new FileFilterPNG300());
		lChooser.addChoosableFileFilter(new FileFilterPNG600());


		int returnVal = lChooser.showSaveDialog(compToOverlap);
		File lFile = null;
		if(returnVal == JFileChooser.APPROVE_OPTION) {
			ExportPictureType lExportPictureType = null;
			lFile = lChooser.getSelectedFile();
			if (lChooser.getFileFilter() instanceof FileFilterJPG300)
			{
				lExportPictureType = ExportPictureType.JPG300;
				lFile = checkExtension(lFile,".jpg");
			}
			else if (lChooser.getFileFilter() instanceof FileFilterJPG600)
			{
				lExportPictureType = ExportPictureType.JPG600;
				lFile = checkExtension(lFile,".jpg");
			}
			else if (lChooser.getFileFilter() instanceof FileFilterPNG300)
			{
				lExportPictureType = ExportPictureType.PNG300;
				lFile = checkExtension(lFile,".png");
			}
			else if (lChooser.getFileFilter() instanceof FileFilterPNG600)
			{
				lExportPictureType = ExportPictureType.PNG600;
				lFile = checkExtension(lFile,".png");
			}
			else
			{
			}
			exportComponentImageHelper(lFile, lExportPictureType, comp, rect);
		}
		if(lFile != null) {
			return lFile.getPath();
		}
		else {
			return null;
		}
	}

	private void exportComponentImageHelper(File pFile, ExportPictureType pEPT, Component comp, Rectangle rect) {
		int lDetail;
		String lType;
		if (pEPT == ExportPictureType.JPG600  || pEPT == ExportPictureType.PNG600)
		{
			lDetail = 600;
		}
		else
		{
			lDetail = 300;
		}
		if (pEPT == ExportPictureType.JPG300 || pEPT == ExportPictureType.JPG600)
		{
			lType = "jpg";
		}
		else
		{
			lType = "png";
		}
		BufferedImage lImg = createGraphicData(lDetail,pFile, comp, rect);
		if(lImg != null)
		{
			writeGraphicsFile(lType,pFile,lImg, comp);
		}
	}

	private BufferedImage createGraphicData(int lDetail, File pFile, Component comp, Rectangle rect)
	{
		Color lColor = comp.getBackground();
		comp.setBackground(new Color(255,255,255));

		try
		{
			BufferedImage lImg = new BufferedImage(
					comp.getWidth(),
					comp.getHeight(),
					BufferedImage.TYPE_INT_RGB
			);
			Graphics lGraphics= lImg.getGraphics();		
			Graphics2D lG2D = ((Graphics2D)lGraphics);
			comp.print(lG2D);
			comp.setBackground(lColor);

			//cropping
			lImg = lImg.getSubimage((int) rect.getX(), (int) rect.getY(), 
					(int) rect.getWidth(), (int) rect.getHeight());

			return lImg;
		}
		catch (OutOfMemoryError pOME)
		{
			JOptionPane.showMessageDialog(comp,"Java was not allocated enough memory to export this picture","Run this program with -Xmx256m -Xms64m options",JOptionPane.ERROR_MESSAGE);
		}
		comp.setBackground(lColor);
		return null;
	}

	private void writeGraphicsFile(String lType, File pFile, BufferedImage lImg, Component comp)
	{
		if (lType.equals("jpg"))
		{
			try {
				OutputStream lOut;
				lOut = new FileOutputStream(pFile);
				Iterator<ImageWriter> writers = ImageIO.getImageWritersBySuffix("jpeg");	
				ImageWriter writer = (ImageWriter) writers.next();
				// this is the magic bullet - it needs to output to an object
				// not a file
				ImageOutputStream ios = ImageIO.createImageOutputStream(lOut);
				writer.setOutput(ios);
				ImageWriteParam param = writer.getDefaultWriteParam();
				param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
				param.setCompressionQuality(1.0F);
				writer.write(null, new IIOImage(lImg, null, null), param);
				//writer.write
				ios.close();
				writer.dispose();
			} catch (FileNotFoundException e) {
				e.printStackTrace();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}	
		else if (lType.equals("png"))
		{
			try {
				ImageIO.write(lImg, "png", pFile);
			} catch (IOException e) {
				e.printStackTrace();
			}		
		}
		else
		{
			JOptionPane.showMessageDialog(comp,"You have not chosen a valid file type","I have no idea how you got here!",JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * @param lFile The passed in file Object
	 * @param pExtension The extension required.
	 * @return a File Object with the correct extension
	 */
	private File checkExtension(File lFile, String pExtension) {
		if (lFile.getName().endsWith(pExtension))
		{
			// do nothing
		}
		else
		{	
			if (lFile.getName().indexOf(".") == -1)
			{
				lFile = new File(lFile.getPath() + pExtension);
			}
			else
			{
				lFile = new File(lFile.getPath().substring(0,lFile.getPath().indexOf(".")) + pExtension);
			}
		}
		return lFile;
	}


}
